@@ -18,6 +18,7 @@
 //!   let minus = ByteSize::tb(100) - ByteSize::gb(4);
 //!   print!("{} bytes", minus.as_usize());
 //! }
+//! ```
 //!
 //! It also provides its human readable string as follows:
 //!
@@ -56,7 +57,7 @@ pub static TIB: usize = 1099511627776;
 /// bytes size for 1 pebibyte
 pub static PIB: usize = 1125899906842624;
 
-#[derive(Debug, Copy, Clone)]
+#[derive(Debug, Copy, Clone, PartialEq, PartialOrd, Eq, Ord)]
 /// Byte size representation
 pub struct ByteSize {
   size: usize
@@ -129,14 +130,17 @@ impl ByteSize {
     let unit = if si { KIB } else { KB };
 
     if self.size < unit {
-      {format!("{} B", self.size)}
+      format!("{} B", self.size)
     } else {
-      let exp = ((self.size as f64).ln() / (if si {LN_KIB} else {LN_KB})) as usize;
+      let mut exp = ((self.size as f64).ln() / (if si {LN_KIB} else {LN_KB})) as usize;
+      if exp == 0 {
+        exp = 1;
+      }
 
       if si {
-        format!("{} {}iB",(self.size / num::pow(unit, exp)), UNITS_SI.as_bytes()[exp - 1] as char)
+        format!("{} {}iB", (self.size / num::pow(unit, exp)), UNITS_SI.as_bytes()[exp - 1] as char)
       } else {
-        format!("{} {}B",(self.size / num::pow(unit, exp)), UNITS.as_bytes()[exp - 1] as char)
+        format!("{} {}B", (self.size / num::pow(unit, exp)), UNITS.as_bytes()[exp - 1] as char)
       }
     }
   }
@@ -144,7 +148,7 @@ impl ByteSize {
 
 static UNITS:    &'static str = "KMGTPE";
 static UNITS_SI: &'static str = "kMGTPE";
-static LN_KB: f64 = 6.931471806; // ln 1024
+static LN_KB:  f64 = 6.931471806; // ln 1024
 static LN_KIB: f64 = 6.907755279; // ln 1000
 
 impl Display for ByteSize {
@@ -207,67 +211,87 @@ impl Div<usize> for ByteSize {
   }
 }
 
-#[test]
-fn test_arithmetic() {
-  let x = ByteSize::mb(1);
-  let y = ByteSize::kb(100);
 
-  assert_eq!(
-    (x + y).as_usize(),
-    1100000
-  );
-  assert_eq!(
-    (x + (100*1000)).as_usize(),
-    1100000
-  );
+#[cfg(test)]
+mod tests {
+  use super::*;
 
-  assert_eq!(
-    (x - y).as_usize(),
-    900000
-  );
-  assert_eq!(
-    (x - (100*1000)).as_usize(),
-    900000
-  );
-}
-
-#[allow(dead_code)]
-fn assert_display(expected: &str, b: ByteSize) {
-  assert_eq!(expected, format!("{}", b));
-}
-
-#[test]
-fn test_display() {
-  assert_display("215 B", ByteSize::b(215));
-  assert_display("301 KB", ByteSize::kb(301));
-  assert_display("419 MB", ByteSize::mb(419));
-  assert_display("518 GB", ByteSize::gb(518));
-  assert_display("815 TB", ByteSize::tb(815));
-  assert_display("609 PB", ByteSize::pb(609));
-}
-
-#[allow(dead_code)]
-fn assert_to_string(expected: &str, b: ByteSize, si: bool) {
-  assert_eq!(expected.to_string(), b.to_string(si));
-}
-
-#[test]
-fn test_to_string() {
-  assert_to_string("215 B", ByteSize::b(215), true);
-  assert_to_string("215 B", ByteSize::b(215), false);
-
-  assert_to_string("293 kiB", ByteSize::kb(301), true);
-  assert_to_string("301 KB", ByteSize::kb(301), false);
-
-  assert_to_string("399 MiB", ByteSize::mb(419), true);
-  assert_to_string("419 MB", ByteSize::mb(419), false);
-
-  assert_to_string("482 GiB", ByteSize::gb(518), true);
-  assert_to_string("518 GB", ByteSize::gb(518), false);
-
-  assert_to_string("741 TiB", ByteSize::tb(815), true);
-  assert_to_string("815 TB", ByteSize::tb(815), false);
-
-  assert_to_string("540 PiB", ByteSize::pb(609), true);
-  assert_to_string("609 PB", ByteSize::pb(609), false);
+  #[test]
+  fn test_arithmetic() {
+    let x = ByteSize::mb(1);
+    let y = ByteSize::kb(100);
+
+    assert_eq!(
+      (x + y).as_usize(),
+      1100000
+    );
+    assert_eq!(
+      (x + (100*1000)).as_usize(),
+      1100000
+    );
+
+    assert_eq!(
+      (x - y).as_usize(),
+      900000
+    );
+    assert_eq!(
+      (x - (100*1000)).as_usize(),
+      900000
+    );
+  }
+
+  #[test]
+  fn test_comparison() {
+    assert!(ByteSize::mb(1) == ByteSize::kb(1000));
+    assert!(ByteSize::mib(1) == ByteSize::kib(1024));
+    assert!(ByteSize::mb(1) != ByteSize::kib(1024));
+    assert!(ByteSize::mb(1) < ByteSize::kib(1024));
+    assert!(ByteSize::b(0) < ByteSize::tib(1));
+  }
+
+  fn assert_display(expected: &str, b: ByteSize) {
+    assert_eq!(expected, format!("{}", b));
+  }
+
+  #[test]
+  fn test_display() {
+    assert_display("215 B", ByteSize::b(215));
+    assert_display("1 KB", ByteSize::kb(1));
+    assert_display("301 KB", ByteSize::kb(301));
+    assert_display("419 MB", ByteSize::mb(419));
+    assert_display("518 GB", ByteSize::gb(518));
+    assert_display("815 TB", ByteSize::tb(815));
+    assert_display("609 PB", ByteSize::pb(609));
+  }
+
+  fn assert_to_string(expected: &str, b: ByteSize, si: bool) {
+    assert_eq!(expected.to_string(), b.to_string(si));
+  }
+
+  #[test]
+  fn test_to_string() {
+    assert_to_string("215 B", ByteSize::b(215), true);
+    assert_to_string("215 B", ByteSize::b(215), false);
+
+    assert_to_string("1 kiB", ByteSize::kib(1), true);
+    assert_to_string("1 KB", ByteSize::kib(1), false);
+
+    assert_to_string("293 kiB", ByteSize::kb(301), true);
+    assert_to_string("301 KB", ByteSize::kb(301), false);
+
+    assert_to_string("1 MiB", ByteSize::mib(1), true);
+    assert_to_string("1048 KB", ByteSize::mib(1), false);
+
+    assert_to_string("399 MiB", ByteSize::mb(419), true);
+    assert_to_string("419 MB", ByteSize::mb(419), false);
+
+    assert_to_string("482 GiB", ByteSize::gb(518), true);
+    assert_to_string("518 GB", ByteSize::gb(518), false);
+
+    assert_to_string("741 TiB", ByteSize::tb(815), true);
+    assert_to_string("815 TB", ByteSize::tb(815), false);
+
+    assert_to_string("540 PiB", ByteSize::pb(609), true);
+    assert_to_string("609 PB", ByteSize::pb(609), false);
+  }
 }
